"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Subscription = void 0;
const vue_1 = require("vue");
const Function_1 = require("./Function");
class Subscription {
    constructor() {
        Object.defineProperty(this, "callbacks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    addSubscription(callback, options = { detached: false, onCleanup: Function_1.noop }) {
        const callbackInfo = {
            callback,
            options,
            disposer: () => {
                const idx = this.callbacks.indexOf(callbackInfo);
                if (idx > -1) {
                    this.callbacks.splice(idx, 1);
                    options.onCleanup();
                }
            }
        };
        this.callbacks.push(callbackInfo);
        if (!options.detached && (0, vue_1.getCurrentScope)()) {
            (0, vue_1.onScopeDispose)(callbackInfo.disposer);
        }
        return callbackInfo;
    }
    subscribe(callback, options = { detached: false, onCleanup: Function_1.noop }) {
        const info = this.addSubscription(callback, options);
        return info.disposer;
    }
    trigger(...args) {
        this.callbacks.slice().forEach(({ callback }) => {
            callback(...args);
        });
    }
    triggerPiped(...args) {
        let resultArgs = args;
        this.callbacks.slice().forEach(({ callback }) => {
            resultArgs = callback.call(null, ...resultArgs);
        });
        return resultArgs;
    }
}
exports.Subscription = Subscription;
//# sourceMappingURL=Subscription.js.map